/**
   @version 1.10 17 Aug 1998
   @author Cay Horstmann
*/


import java.io.*;
import java.util.*;

class ObjectRefTest
{  
   public static void main(String[] args)
   {  
      Employee harry = new Employee("Harry Hacker", 50000,
         1989, 10, 1);
      Manager boss = new Manager("Carl Cracker", 80000,
         1987, 12, 15);
      boss.setSecretary(harry);

      Employee[] staff = new Employee[3];

      staff[0] = boss;
      staff[1] = harry;
      staff[2] = new Employee("Tony Tester", 40000,
         1990, 3, 15);
      
      try
      {  
         // save all employee records to the file employee.dat
         ObjectOutputStream out = new ObjectOutputStream(new 
            FileOutputStream("employee.dat"));
         out.writeObject(staff);
         out.close();

         // retrieve all records into a new array
         ObjectInputStream in =  new ObjectInputStream(new 
            FileInputStream("employee.dat"));
         Employee[] newStaff = (Employee[])in.readObject();
         in.close();

         // raise secretary's salary
         newStaff[1].raiseSalary(10); 

         // print the newly read employee records
         for (int i = 0; i < newStaff.length; i++)
            System.out.println(newStaff[i]);
      }
      catch (Exception e)
      {  
         e.printStackTrace(); 
      }
   }
}

class Employee implements Serializable
{
   public Employee() {}

   public Employee(String n, double s, 
      int year, int month, int day)
   {  
      name = n;
      salary = s;
      GregorianCalendar calendar
         = new GregorianCalendar(year, month - 1, day);
         // GregorianCalendar uses 0 for January
      hireDay = calendar.getTime();
   }

   public String getName()
   {  
      return name;
   }

   public double getSalary()
   {  
      return salary;
   }

   public Date getHireDay()
   {  
      return hireDay;
   }

   public void raiseSalary(double byPercent)
   {  
      double raise = salary * byPercent / 100;
      salary += raise;
   }

   public String toString()
   {  
      return getClass().getName()
         + "[name=" + name
         + ",salary=" + salary
         + ",hireDay=" + hireDay
         + "]";
   }

   private String name;
   private double salary;
   private Date hireDay;
}

class Manager extends Employee
{  
   /**
      Constructs a Manager without a secretary
      @param n the employee's name
      @param s the salary
      @param year the hire year
      @param month the hire month
      @param day the hire day
   */
   public Manager(String n, double s,
      int year, int month, int day)
   {  
      super(n, s, year, month, day);
      secretary = null;
   }

   /**
      Assigns a secretary to the manager.
      @param s the secretary
   */
   public void setSecretary(Employee s)
   {  
      secretary = s;
   }

   public String toString()
   {
      return super.toString()
        + "[secretary=" + secretary
        + "]";
   }

   private Employee secretary;
}

